From 56c942c2a0b6eea76b81b108e866e3559192f9fe Mon Sep 17 00:00:00 2001
From: Amul Sul <amul.sul@enterprisedb.com>
Date: Wed, 14 Dec 2022 11:40:11 +0530
Subject: [PATCH v1 20/25] Change pg_snapshot_in to allow non-throw error
 reporting

---
 src/backend/utils/adt/xid8funcs.c         |  7 +++----
 src/test/regress/expected/type_sanity.out | 12 ++++++++++++
 src/test/regress/sql/type_sanity.sql      |  3 +++
 3 files changed, 18 insertions(+), 4 deletions(-)

diff --git a/src/backend/utils/adt/xid8funcs.c b/src/backend/utils/adt/xid8funcs.c
index d8e40b3b969..2093776809f 100644
--- a/src/backend/utils/adt/xid8funcs.c
+++ b/src/backend/utils/adt/xid8funcs.c
@@ -285,7 +285,7 @@ buf_finalize(StringInfo buf)
  * parse snapshot from cstring
  */
 static pg_snapshot *
-parse_snapshot(const char *str)
+parse_snapshot(const char *str, Node *escontext)
 {
 	FullTransactionId xmin;
 	FullTransactionId xmax;
@@ -341,11 +341,10 @@ parse_snapshot(const char *str)
 	return buf_finalize(buf);
 
 bad_format:
-	ereport(ERROR,
+	ereturn(escontext, NULL,
 			(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
 			 errmsg("invalid input syntax for type %s: \"%s\"",
 					"pg_snapshot", str_start)));
-	return NULL;				/* keep compiler quiet */
 }
 
 /*
@@ -447,7 +446,7 @@ pg_snapshot_in(PG_FUNCTION_ARGS)
 	char	   *str = PG_GETARG_CSTRING(0);
 	pg_snapshot *snap;
 
-	snap = parse_snapshot(str);
+	snap = parse_snapshot(str, fcinfo->context);
 
 	PG_RETURN_POINTER(snap);
 }
diff --git a/src/test/regress/expected/type_sanity.out b/src/test/regress/expected/type_sanity.out
index d20bf8b7c5d..7a58f1cd8b9 100644
--- a/src/test/regress/expected/type_sanity.out
+++ b/src/test/regress/expected/type_sanity.out
@@ -880,3 +880,15 @@ SELECT pg_input_error_message('1', 'pg_ndistinct');
  cannot accept a value of type pg_ndistinct
 (1 row)
 
+SELECT pg_input_is_valid('12:34:xyz', 'pg_snapshot');
+ pg_input_is_valid 
+-------------------
+ f
+(1 row)
+
+SELECT pg_input_error_message('12:34:xyz', 'pg_snapshot');
+                 pg_input_error_message                 
+--------------------------------------------------------
+ invalid input syntax for type pg_snapshot: "12:34:xyz"
+(1 row)
+
diff --git a/src/test/regress/sql/type_sanity.sql b/src/test/regress/sql/type_sanity.sql
index 9bbec68082c..4dd133c8eb2 100644
--- a/src/test/regress/sql/type_sanity.sql
+++ b/src/test/regress/sql/type_sanity.sql
@@ -616,3 +616,6 @@ SELECT pg_input_error_message('1', 'pg_mcv_list');
 
 SELECT pg_input_is_valid('1', 'pg_ndistinct');
 SELECT pg_input_error_message('1', 'pg_ndistinct');
+
+SELECT pg_input_is_valid('12:34:xyz', 'pg_snapshot');
+SELECT pg_input_error_message('12:34:xyz', 'pg_snapshot');
-- 
2.18.0

